@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2022 IBM Corporation. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -109,7 +110,7 @@ public class AsOfClause implements Serializable {
         } else {
             ConversionManager converter = ConversionManager.getDefaultManager();
             value = converter.convertObject(value, ClassConstants.TIMESTAMP);
-            printer.printPrimitive(value);
+            printer.printPrimitive(value, true);
         }
         printer.printString(")");
     }
