@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1998, 2019 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2019 IBM Corporation. All rights reserved.
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2022 IBM Corporation. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -347,13 +347,15 @@ public class FunctionExpression extends BaseExpression {
      * INTERNAL:
      */
     public void initializePlatformOperator(DatabasePlatform platform) {
-        if (this.operator.isComplete()) {
-            platformOperator = this.operator;
-            return;
-        }
+        // First, check that the platform operator doesn't override the operator behavior
         platformOperator = platform.getOperator(this.operator.getSelector());
         if (platformOperator == null) {
-            throw QueryException.invalidOperator(this.operator.toString());
+            // If the platform doesn't specifically override, fallback on the internal operator
+            // This operator should be either user-defined or one from ExpressionOperator.initializeInternalOperators.
+            platformOperator = this.operator;
+            if (platformOperator == null) {
+                throw QueryException.invalidOperator(this.operator);
+            }
         }
     }
 
@@ -558,7 +560,10 @@ public class FunctionExpression extends BaseExpression {
      */
     @Override
     public void printSQL(ExpressionSQLPrinter printer) {
-        // If all children are parameters, some databases don't allow binding.
+        /*
+         * If this ExpressionOperator does not support binding, and the platform allows,
+         * then disable binding for the whole query
+         */
         if (printer.getPlatform().isDynamicSQLRequiredForFunctions() && !this.children.isEmpty()) {
             boolean allParams = true;
             for (Iterator<Expression> iterator = this.children.iterator(); iterator.hasNext(); ) {
