@@ -1,15 +1,17 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
 package org.eclipse.persistence.internal.oxm;
 
 import java.util.List;
@@ -53,7 +55,7 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
         super();
         this.xmlCompositeCollectionMapping = xmlCompositeCollectionMapping;
     }
-        
+
     public XMLCompositeCollectionMappingNodeValue(CompositeCollectionMapping xmlCompositeCollectionMapping, boolean isInverse) {
         this(xmlCompositeCollectionMapping);
         this.isInverseReference = isInverse;
@@ -74,57 +76,59 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
                 return false;
             }
         }
-        
+
         CoreContainerPolicy cp = getContainerPolicy();
         int size = marshalRecord.getCycleDetectionStack().size();
         // when writing the collection need to see if any of the objects we are
         // writing are in the parent collection inverse ref
         if ((isInverseReference || xmlCompositeCollectionMapping.getInverseReferenceMapping() != null) && size >= 2) {
-        	Object owner = marshalRecord.getCycleDetectionStack().get(size - 2);
-        	try {
-        		if (cp.contains(owner, collection, session)) {
-        			return false;
-        		}
-        	} catch (ClassCastException e) {
-        		// For Bug #416875
-        	}
+            Object owner = marshalRecord.getCycleDetectionStack().get(size - 2);
+            try {
+                if (cp.contains(owner, collection, session)) {
+                    return false;
+                }
+            } catch (ClassCastException e) {
+                // For Bug #416875
+            }
         }
-        Object iterator = cp.iteratorFor(collection);        
+        Object iterator = cp.iteratorFor(collection);
         if (null != iterator && cp.hasNext(iterator)) {
-        	XPathFragment groupingFragment = marshalRecord.openStartGroupingElements(namespaceResolver);
-        	marshalRecord.closeStartGroupingElements(groupingFragment);
-        } else {
-        	return marshalRecord.emptyCollection(xPathFragment, namespaceResolver, xmlCompositeCollectionMapping.getWrapperNullPolicy() != null);
+            XPathFragment groupingFragment = marshalRecord.openStartGroupingElements(namespaceResolver);
+            marshalRecord.closeStartGroupingElements(groupingFragment);
+            int valueSize = cp.sizeFor(collection);
+            if(marshalRecord.getMarshaller().isApplicationJSON() && (valueSize > 1 || !marshalRecord.getMarshaller().isReduceAnyArrays())) {
+                marshalRecord.startCollection();
+            }
+            do {
+                Object objectValue = cp.next(iterator, session);
+                marshalSingleValue(xPathFragment, marshalRecord, object, objectValue, session, namespaceResolver, ObjectMarshalContext.getInstance());
+            } while (cp.hasNext(iterator));
+            if(marshalRecord.getMarshaller().isApplicationJSON() && (valueSize > 1 || !marshalRecord.getMarshaller().isReduceAnyArrays())) {
+                marshalRecord.endCollection();
+            }
+            return true;
         }
-                        
-        marshalRecord.startCollection();
-        iterator = cp.iteratorFor(collection);
-        while (cp.hasNext(iterator)) {
-            Object objectValue = cp.next(iterator, session);
-            marshalSingleValue(xPathFragment, marshalRecord, object, objectValue, session, namespaceResolver, ObjectMarshalContext.getInstance());
-        }
-        marshalRecord.endCollection();
-        return true;
+        return marshalRecord.emptyCollection(xPathFragment, namespaceResolver, xmlCompositeCollectionMapping.getWrapperNullPolicy() != null);
     }
 
     public boolean startElement(XPathFragment xPathFragment, UnmarshalRecord unmarshalRecord, Attributes atts) {
-        try {        	
-        	Descriptor xmlDescriptor = (Descriptor) xmlCompositeCollectionMapping.getReferenceDescriptor();        	
+        try {
+            Descriptor xmlDescriptor = (Descriptor) xmlCompositeCollectionMapping.getReferenceDescriptor();
             if (xmlDescriptor == null) {
-            	xmlDescriptor = findReferenceDescriptor(xPathFragment, unmarshalRecord, atts, xmlCompositeCollectionMapping, xmlCompositeCollectionMapping.getKeepAsElementPolicy());
-                
-            	if (xmlDescriptor == null) {
-            		if (unmarshalRecord.getXMLReader().isNullRepresentedByXsiNil(xmlCompositeCollectionMapping.getNullPolicy())) {
-            			if (unmarshalRecord.isNil()) {
+                xmlDescriptor = findReferenceDescriptor(xPathFragment, unmarshalRecord, atts, xmlCompositeCollectionMapping, xmlCompositeCollectionMapping.getKeepAsElementPolicy());
+
+                if (xmlDescriptor == null) {
+                    if (unmarshalRecord.getXMLReader().isNullRepresentedByXsiNil(xmlCompositeCollectionMapping.getNullPolicy())) {
+                        if (unmarshalRecord.isNil()) {
                             return true;
-                		}
-                    } else if (xmlCompositeCollectionMapping.getNullPolicy().valueIsNull(atts)) { 
-                    	getContainerPolicy().addInto(null, unmarshalRecord.getContainerInstance(this), unmarshalRecord.getSession());
-                    	return true;
+                        }
+                    } else if (xmlCompositeCollectionMapping.getNullPolicy().valueIsNull(atts)) {
+                        getContainerPolicy().addInto(null, unmarshalRecord.getContainerInstance(this), unmarshalRecord.getSession());
+                        return true;
                     }
-            		if (xmlCompositeCollectionMapping.getField() != null) {
+                    if (xmlCompositeCollectionMapping.getField() != null) {
                         //try leaf element type
-            			QName leafType = ((Field) xmlCompositeCollectionMapping.getField()).getLastXPathFragment().getLeafElementType();
+                        QName leafType = ((Field) xmlCompositeCollectionMapping.getField()).getLastXPathFragment().getLeafElementType();
                         if (leafType != null) {
                             XPathFragment frag = new XPathFragment();
                             frag.setNamespaceAware(unmarshalRecord.isNamespaceAware());
@@ -138,16 +142,16 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
                                     xpath = prefix + Constants.COLON + xpath;
                                 }
                             }
-                            frag.setXPath(xpath);  
+                            frag.setXPath(xpath);
                             Context xmlContext = unmarshalRecord.getUnmarshaller().getContext();
                             xmlDescriptor = xmlContext.getDescriptorByGlobalType(frag);
                         }
                     }
-                } 
-                    
+                }
+
                 UnmarshalKeepAsElementPolicy policy = xmlCompositeCollectionMapping.getKeepAsElementPolicy();
-                if (policy != null && ((xmlDescriptor == null && policy.isKeepUnknownAsElement()) || policy.isKeepAllAsElement())) {                    
-                    if (unmarshalRecord.getTypeQName() != null) {                    	
+                if (policy != null && ((xmlDescriptor == null && policy.isKeepUnknownAsElement()) || policy.isKeepAllAsElement())) {
+                    if (unmarshalRecord.getTypeQName() != null) {
                         Class theClass = unmarshalRecord.getConversionManager().javaType(unmarshalRecord.getTypeQName());
                         if (theClass == null) {
                             setupHandlerForKeepAsElementPolicy(unmarshalRecord, xPathFragment, atts);
@@ -157,27 +161,27 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
                         setupHandlerForKeepAsElementPolicy(unmarshalRecord, xPathFragment, atts);
                         return true;
                     }
-                }          
+                }
             }
 
             AbstractNullPolicy nullPolicy = xmlCompositeCollectionMapping.getNullPolicy();
             if (nullPolicy.isNullRepresentedByEmptyNode()) {
                 String qnameString = xPathFragment.getLocalName();
                 if (xPathFragment.getPrefix() != null) {
-                	qnameString = xPathFragment.getPrefix() + Constants.COLON + qnameString;
+                    qnameString = xPathFragment.getPrefix() + Constants.COLON + qnameString;
                 }
                 if (null != xmlDescriptor) {
-                	// Process null capable value
+                    // Process null capable value
                     CompositeCollectionMappingContentHandler aHandler = new CompositeCollectionMappingContentHandler(//
-                    		unmarshalRecord, this, xmlCompositeCollectionMapping, atts, xPathFragment, xmlDescriptor);
+                            unmarshalRecord, this, xmlCompositeCollectionMapping, atts, xPathFragment, xmlDescriptor);
                     // Send control to the handler
                     aHandler.startElement(xPathFragment.getNamespaceURI(), xPathFragment.getLocalName(), qnameString, atts);
                     XMLReader xmlReader = unmarshalRecord.getXMLReader();
                     xmlReader.setContentHandler(aHandler);
                     xmlReader.setLexicalHandler(aHandler);
                 }
-            } else if (!(unmarshalRecord.getXMLReader().isNullRecord(nullPolicy, atts, unmarshalRecord))) {            	
-            	Field xmlFld = (Field) this.xmlCompositeCollectionMapping.getField();
+            } else if (!(unmarshalRecord.getXMLReader().isNullRecord(nullPolicy, atts, unmarshalRecord))) {
+                Field xmlFld = (Field) this.xmlCompositeCollectionMapping.getField();
                 if (xmlFld.hasLastXPathFragment()) {
                     unmarshalRecord.setLeafElementType(xmlFld.getLastXPathFragment().getLeafElementType());
                 }
@@ -193,22 +197,22 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
         Object collection = unmarshalRecord.getContainerInstance(this);
         endElement(xPathFragment, unmarshalRecord, collection);
     }
-    
+
     public void endElement(XPathFragment xPathFragment, UnmarshalRecord unmarshalRecord, Object collection) {
-        if(unmarshalRecord.isNil() && unmarshalRecord.getXMLReader().isNullRepresentedByXsiNil(xmlCompositeCollectionMapping.getNullPolicy()) && 
-        		(unmarshalRecord.getChildRecord() == null)){
-        	if(unmarshalRecord.getXMLReader().isInCollection()){
-        		unmarshalRecord.addAttributeValue(this, null);
-        	}else{
-        		unmarshalRecord.setAttributeValueNull(this);
-        	}
-            unmarshalRecord.resetStringBuffer();        	
+        if(unmarshalRecord.isNil() && unmarshalRecord.getXMLReader().isNullRepresentedByXsiNil(xmlCompositeCollectionMapping.getNullPolicy()) &&
+                (unmarshalRecord.getChildRecord() == null)){
+            if(unmarshalRecord.getXMLReader().isInCollection()){
+                unmarshalRecord.addAttributeValue(this, null);
+            }else{
+                unmarshalRecord.setAttributeValueNull(this);
+            }
+            unmarshalRecord.resetStringBuffer();
             return;
         }
         if (null == unmarshalRecord.getChildRecord()) {
                SAXFragmentBuilder builder = unmarshalRecord.getFragmentBuilder();
-               UnmarshalKeepAsElementPolicy keepAsElementPolicy = xmlCompositeCollectionMapping.getKeepAsElementPolicy();        
-                              
+               UnmarshalKeepAsElementPolicy keepAsElementPolicy = xmlCompositeCollectionMapping.getKeepAsElementPolicy();
+
                if (null != keepAsElementPolicy && (keepAsElementPolicy.isKeepUnknownAsElement() || keepAsElementPolicy.isKeepAllAsElement()) && builder.getNodes().size() > 1) {
                    if(unmarshalRecord.getTypeQName() != null){
                        Class theClass = unmarshalRecord.getConversionManager().javaType(unmarshalRecord.getTypeQName());
@@ -218,7 +222,7 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
                            return;
                        }
                    }
-            	   if(builder.getNodes().size() > 1) {
+                   if(builder.getNodes().size() > 1) {
                        setOrAddAttributeValueForKeepAsElement(builder, xmlCompositeCollectionMapping, xmlCompositeCollectionMapping, unmarshalRecord, true, collection);
                        return;
                    }
@@ -235,7 +239,7 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
         InverseReferenceMapping inverseReferenceMapping = xmlCompositeCollectionMapping.getInverseReferenceMapping();
         if(null != inverseReferenceMapping) {
             if(inverseReferenceMapping.getContainerPolicy() == null) {
-            	Object currentValue = inverseReferenceMapping.getAttributeAccessor().getAttributeValueFromObject(objectValue);
+                Object currentValue = inverseReferenceMapping.getAttributeAccessor().getAttributeValueFromObject(objectValue);
                 if( !isInverseReference || (currentValue == null && isInverseReference)) {
                    inverseReferenceMapping.getAttributeAccessor().setAttributeValueInObject(objectValue, unmarshalRecord.getCurrentObject());
                 }
@@ -248,7 +252,7 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
                 inverseReferenceMapping.getContainerPolicy().addInto(unmarshalRecord.getCurrentObject(), backpointerContainer, unmarshalRecord.getSession());
             }
         }
-        
+
         // convert the value - if necessary
         objectValue = xmlCompositeCollectionMapping.convertDataValueToObjectValue(objectValue, unmarshalRecord.getSession(), unmarshalRecord.getUnmarshaller());
         unmarshalRecord.addAttributeValue(this, objectValue, collection);
@@ -268,42 +272,42 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
     public CoreContainerPolicy getContainerPolicy() {
         return xmlCompositeCollectionMapping.getContainerPolicy();
     }
-    
+
     public boolean isContainerValue() {
         return true;
     }
-    
-	public boolean marshalSingleValue(XPathFragment xPathFragment, MarshalRecord marshalRecord, Object object, Object value, CoreAbstractSession session, NamespaceResolver namespaceResolver, MarshalContext marshalContext) {
-      
+
+    public boolean marshalSingleValue(XPathFragment xPathFragment, MarshalRecord marshalRecord, Object object, Object value, CoreAbstractSession session, NamespaceResolver namespaceResolver, MarshalContext marshalContext) {
+
         Marshaller marshaller = marshalRecord.getMarshaller();
         // convert the value - if necessary
         boolean isNil = false;
         if (value instanceof Root) {
-        	isNil = ((Root) value).nil;
-        	value = ((Root) value).getObject();
+            isNil = ((Root) value).nil;
+            value = ((Root) value).getObject();
         }
-        
+
         value = xmlCompositeCollectionMapping.convertObjectValueToDataValue(value, session, marshaller);
         if (null == value) {
-        	   return xmlCompositeCollectionMapping.getNullPolicy().compositeObjectMarshal(xPathFragment, marshalRecord, object, session, namespaceResolver);
+               return xmlCompositeCollectionMapping.getNullPolicy().compositeObjectMarshal(xPathFragment, marshalRecord, object, session, namespaceResolver);
         }
         Descriptor descriptor = (Descriptor)xmlCompositeCollectionMapping.getReferenceDescriptor();
         if(descriptor == null){
-        	descriptor = (Descriptor) session.getDescriptor(value.getClass());
+            descriptor = (Descriptor) session.getDescriptor(value.getClass());
         }else if(descriptor.hasInheritance()){
-        	Class objectValueClass = value.getClass();
-        	if(!(objectValueClass == descriptor.getJavaClass())){
-        		descriptor = (Descriptor) session.getDescriptor(objectValueClass);
-        	}
+            Class objectValueClass = value.getClass();
+            if(!(objectValueClass == descriptor.getJavaClass())){
+                descriptor = (Descriptor) session.getDescriptor(objectValueClass);
+            }
         }
-        
+
         UnmarshalKeepAsElementPolicy keepAsElementPolicy = xmlCompositeCollectionMapping.getKeepAsElementPolicy();
         if (null != keepAsElementPolicy && (keepAsElementPolicy.isKeepUnknownAsElement() || keepAsElementPolicy.isKeepAllAsElement()) && value instanceof org.w3c.dom.Node) {
             marshalRecord.node((org.w3c.dom.Node) value, marshalRecord.getNamespaceResolver());
             return true;
         }
-        
-        if(descriptor != null){                    
+
+        if(descriptor != null){
             marshalRecord.beforeContainmentMarshal(value);
 
             ObjectBuilder objectBuilder = (ObjectBuilder)descriptor.getObjectBuilder();
@@ -314,16 +318,16 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
             if(item != null) {
                 if(item.getGroups() != null) {
                     nestedGroup = item.getGroup(descriptor.getJavaClass());
-                } 
+                }
                 if(nestedGroup == null) {
                     nestedGroup = item.getGroup() == null?XMLRecord.DEFAULT_ATTRIBUTE_GROUP:item.getGroup();
                 }
             }
             marshalRecord.pushAttributeGroup(nestedGroup);
-            
+
             xPathNode.startElement(marshalRecord, xPathFragment, object, session, namespaceResolver, objectBuilder, value);
             if (isNil) {
-            	marshalRecord.nilSimple(namespaceResolver);
+                marshalRecord.nilSimple(namespaceResolver);
             }
 
             List extraNamespaces = objectBuilder.addExtraNamespacesToNamespaceResolver(descriptor, marshalRecord, session,true, false);
@@ -335,16 +339,16 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
             marshalRecord.afterContainmentMarshal(object, value);
             marshalRecord.popAttributeGroup();
             marshalRecord.endElement(xPathFragment, namespaceResolver);
-            marshalRecord.removeExtraNamespacesFromNamespaceResolver(extraNamespaces, session);    
-        } else {            
-            if(Constants.UNKNOWN_OR_TRANSIENT_CLASS.equals(xmlCompositeCollectionMapping.getReferenceClassName())){                
-                throw XMLMarshalException.descriptorNotFoundInProject(value.getClass().getName());                            
+            marshalRecord.removeExtraNamespacesFromNamespaceResolver(extraNamespaces, session);
+        } else {
+            if(Constants.UNKNOWN_OR_TRANSIENT_CLASS.equals(xmlCompositeCollectionMapping.getReferenceClassName())){
+                throw XMLMarshalException.descriptorNotFoundInProject(value.getClass().getName());
             }
             xPathNode.startElement(marshalRecord, xPathFragment, object, session, namespaceResolver, null, value);
-            
+
             QName schemaType = ((Field) xmlCompositeCollectionMapping.getField()).getSchemaTypeForValue(value, session);
             updateNamespaces(schemaType, marshalRecord,((Field)xmlCompositeCollectionMapping.getField()));
-            marshalRecord.characters(schemaType, value, null, false);            
+            marshalRecord.characters(schemaType, value, null, false);
             marshalRecord.endElement(xPathFragment, namespaceResolver);
         }
         return true;
@@ -355,28 +359,28 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
     }
 
     protected void setOrAddAttributeValue(UnmarshalRecord unmarshalRecord, Object value, XPathFragment xPathFragment, Object collection){
-        unmarshalRecord.addAttributeValue(this, value, collection);            	
+        unmarshalRecord.addAttributeValue(this, value, collection);
     }
 
     public boolean getReuseContainer() {
         return xmlCompositeCollectionMapping.getReuseContainer();
     }
-    
+
     /**
      *  INTERNAL:
-     *  Used to track the index of the corresponding containerInstance in the containerInstances Object[] on UnmarshalRecord 
-     */  
+     *  Used to track the index of the corresponding containerInstance in the containerInstances Object[] on UnmarshalRecord
+     */
     public void setIndex(int index){
-    	this.index = index;
+        this.index = index;
     }
-    
+
     /**
      * INTERNAL:
      * Set to track the index of the corresponding containerInstance in the containerInstances Object[] on UnmarshalRecord
-     * Set during TreeObjectBuilder initialization 
+     * Set during TreeObjectBuilder initialization
      */
     public int getIndex(){
-    	return index;
+        return index;
     }
 
     /**
@@ -394,4 +398,4 @@ public class XMLCompositeCollectionMappingNodeValue extends XMLRelationshipMappi
         return true;
     }
 
-}
\ No newline at end of file
+}
