@@ -1,30 +1,41 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
 package org.eclipse.persistence.internal.sessions.remote;
 
-import java.util.*;
-import java.rmi.server.*;
+import java.rmi.server.ObjID;
+import java.util.Enumeration;
+import java.util.Hashtable;
+import java.util.IdentityHashMap;
+import java.util.Map;
+import java.util.Vector;
 
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.indirection.ValueHolderInterface;
-import org.eclipse.persistence.internal.descriptors.*;
-import org.eclipse.persistence.queries.*;
-import org.eclipse.persistence.internal.queries.*;
+import org.eclipse.persistence.internal.descriptors.DescriptorIterator;
+import org.eclipse.persistence.internal.helper.ClassConstants;
+import org.eclipse.persistence.internal.identitymaps.CacheKey;
+import org.eclipse.persistence.internal.queries.CollectionContainerPolicy;
+import org.eclipse.persistence.internal.queries.ContainerPolicy;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
 import org.eclipse.persistence.internal.sessions.UnitOfWorkImpl;
-import org.eclipse.persistence.internal.helper.*;
-import org.eclipse.persistence.internal.identitymaps.CacheKey;
+import org.eclipse.persistence.queries.Cursor;
+import org.eclipse.persistence.queries.CursorPolicy;
+import org.eclipse.persistence.queries.CursoredStream;
+import org.eclipse.persistence.queries.DatabaseQuery;
+import org.eclipse.persistence.queries.ScrollableCursor;
 import org.eclipse.persistence.sessions.coordination.CommandManager;
 
 /**
@@ -36,13 +47,13 @@ public class RemoteSessionController {
 
     /** Used to resolve transaction treading for client-side nested transaction where server uses many worker threads. */
     protected boolean isInTransaction;
-    
+
     /** Used to isolate queries to a unit of work in an early transaction. */
     protected boolean isInEarlyTransaction;
 
     /** This is a either a database session or a client session depending upon the setup. */
     protected AbstractSession session;
-    
+
     /** Use the temporary unit of work to isolate queries after an early transaction. */
     protected UnitOfWorkImpl unitOfWork;
 
@@ -227,11 +238,12 @@ public class RemoteSessionController {
     public Transporter cursoredStreamClose(Transporter remoteCursoredStreamOid) {
         Transporter transporter = new Transporter();
         try {
-            CursoredStream stream = (CursoredStream)getRemoteCursors().get(remoteCursoredStreamOid.getObject());
+            ObjID id = (ObjID) (remoteCursoredStreamOid.getObject());
+            CursoredStream stream = (CursoredStream) getRemoteCursors().get(id);
             if (stream != null) {
                 stream.close();
             }
-            getRemoteCursors().remove(remoteCursoredStreamOid);
+            getRemoteCursors().remove(id);
         } catch (RuntimeException exception) {
             transporter.setException(exception);
         }
@@ -332,7 +344,7 @@ public class RemoteSessionController {
     /**
      * A named query after serialization is executed locally.
      */
-    public Transporter executeNamedQuery(Transporter nameTransporter, Transporter classTransporter, Transporter argumentsTransporter) {        
+    public Transporter executeNamedQuery(Transporter nameTransporter, Transporter classTransporter, Transporter argumentsTransporter) {
         Transporter transporter = new Transporter();
 
         try {
