@@ -1,15 +1,20 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2016 Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2018 IBM Corporation. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     11/07/2017 - Dalia Abo Sheasha
+//       - 526957 : Split the logging and trace messages
 package org.eclipse.persistence.logging;
 
 import java.io.IOException;
@@ -18,7 +23,6 @@ import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.io.Writer;
-import java.security.AccessController;
 import java.text.DateFormat;
 import java.util.Date;
 
@@ -27,8 +31,8 @@ import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.internal.databaseaccess.Accessor;
 import org.eclipse.persistence.internal.helper.ConversionManager;
 import org.eclipse.persistence.internal.localization.LoggingLocalization;
+import org.eclipse.persistence.internal.localization.TraceLocalization;
 import org.eclipse.persistence.internal.security.PrivilegedAccessHelper;
-import org.eclipse.persistence.internal.security.PrivilegedGetSystemProperty;
 import org.eclipse.persistence.sessions.Session;
 
 /**
@@ -164,10 +168,7 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      * @return The system default log level property value or {@code null} if no such property is set.
      */
     private static String getDefaultLoggingLevelProperty() {
-        return PrivilegedAccessHelper.shouldUsePrivilegedAccess()
-                ? AccessController.doPrivileged(
-                        new PrivilegedGetSystemProperty(PersistenceUnitProperties.LOGGING_LEVEL))
-                : System.getProperty(PersistenceUnitProperties.LOGGING_LEVEL);
+        return PrivilegedAccessHelper.getSystemProperty(PersistenceUnitProperties.LOGGING_LEVEL);
     }
 
     /**
@@ -206,29 +207,7 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      */
     @Override
     public String getLevelString() {
-        int level = getLevel();
-        switch (level) {
-            case OFF:
-                return "OFF";
-            case SEVERE:
-                return "SEVERE";
-            case WARNING:
-                return "WARNING";
-            case INFO:
-                return "INFO";
-            case CONFIG:
-                return "CONFIG";
-            case FINE:
-                return "FINE";
-            case FINER:
-                return "FINER";
-            case FINEST:
-                return "FINEST";
-            case ALL:
-                return "ALL";
-            default:
-                return "INFO";
-    }
+        return LogLevel.toValue(getLevel(), LogLevel.INFO).getName();
     }
 
     /**
@@ -951,7 +930,11 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
     protected String formatMessage(SessionLogEntry entry) {
         String message = entry.getMessage();
         if (entry.shouldTranslate()) {
-            message = LoggingLocalization.buildMessage(message, entry.getParameters());
+            if (entry.getLevel() > FINE) {
+                message = LoggingLocalization.buildMessage(message, entry.getParameters());
+            } else {
+                message = TraceLocalization.buildMessage(message, entry.getParameters(), true);
+            }
         } else {
             //Bug5976657, if there are entry parameters and the string "{0" contained in the message
             //body, we assume it needs to be formatted.
@@ -968,30 +951,8 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      * If value is null or invalid use the default.
      */
     public static int translateStringToLoggingLevel(String loggingLevel) {
-        if (loggingLevel == null){
-            return INFO;
-    }
-        String level = loggingLevel.toUpperCase();
-        if (level.equals("OFF")){
-            return OFF;
-        } else if (level.equals("SEVERE")){
-            return SEVERE;
-        } else if (level.equals("WARNING")){
-            return WARNING;
-        } else if (level.equals("INFO")){
-            return INFO;
-        } else if (level.equals("CONFIG")){
-            return CONFIG;
-        } else if (level.equals("FINE")){
-            return FINE;
-        } else if (level.equals("FINER")){
-            return FINER;
-        } else if (level.equals("FINEST")){
-            return FINEST;
-        } else if (level.equals("ALL")){
-            return ALL;
-        }
-        return INFO;
+        final LogLevel logLevel = LogLevel.toValue(loggingLevel);
+        return logLevel != null ? logLevel.getId() : LogLevel.INFO.getId();
     }
 
     /**
@@ -1158,27 +1119,8 @@ public abstract class AbstractSessionLog implements SessionLog, java.lang.Clonea
      * Translate the string value of the log level to the constant value.
      * If value is null or invalid use the default.
      */
-    public static String translateLoggingLevelToString(int loggingLevel){
-        if (loggingLevel == OFF){
-            return "OFF";
-        } else if (loggingLevel == SEVERE){
-            return "SEVERE";
-        } else if (loggingLevel == WARNING){
-            return "WARNING";
-        } else if (loggingLevel == INFO){
-            return "INFO";
-        } else if (loggingLevel == CONFIG){
-            return "CONFIG";
-        } else if (loggingLevel == FINE){
-            return "FINE";
-        } else if (loggingLevel == FINER){
-            return "FINER";
-        } else if (loggingLevel == FINEST){
-            return "FINEST";
-        } else if (loggingLevel == ALL){
-            return "ALL";
-    }
-        return "INFO";
+    public static String translateLoggingLevelToString(final int loggingLevel) {
+        return LogLevel.toValue(loggingLevel, LogLevel.INFO).getName();
     }
 
 }
