@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1998, 2021 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 1998, 2021 IBM Corporation. All rights reserved.
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022 IBM Corporation. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -49,7 +49,7 @@ import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.expressions.Expression;
 import org.eclipse.persistence.expressions.ExpressionOperator;
 import org.eclipse.persistence.internal.databaseaccess.DatabaseCall;
-import org.eclipse.persistence.internal.databaseaccess.DatasourceCall;
+import org.eclipse.persistence.internal.databaseaccess.DatasourceCall.ParameterType;
 import org.eclipse.persistence.internal.databaseaccess.FieldTypeDefinition;
 import org.eclipse.persistence.internal.expressions.ExpressionSQLPrinter;
 import org.eclipse.persistence.internal.expressions.FunctionExpression;
@@ -202,14 +202,6 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         }
     }
 
-    /**
-     * INTERNAL:
-     * Build operator.
-     */
-    public ExpressionOperator atan2Operator() {
-        return ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Atan2, "ATAN2");
-    }
-
     /**
      * INTERNAL:
      */
@@ -587,25 +579,40 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         addOperator(operatorOuterJoin());
         addOperator(logOperator());
         addOperator(ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Concat, "CONCAT"));
-        addOperator(todayOperator());
-        addOperator(currentDateOperator());
-        addOperator(currentTimeOperator());
+        addOperator(ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.Today, "SYSDATE"));
+        addOperator(ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.CurrentDate, "TO_DATE(CURRENT_DATE)"));
+        addOperator(ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.CurrentTime, "SYSDATE"));
         addOperator(ExpressionOperator.truncateDate());
         addOperator(ExpressionOperator.newTime());
         addOperator(ExpressionOperator.ifNull());
-        addOperator(atan2Operator());
-        addOperator(ExpressionOperator.oracleDateName());
+        addOperator(ExpressionOperator.simpleTwoArgumentFunction(ExpressionOperator.Atan2, "ATAN2"));
+        addOperator(oracleDateName());
         addOperator(operatorLocate());
         addOperator(operatorLocate2());
         addOperator(regexpOperator());
         addOperator(exceptOperator());
     }
 
+    /**
+     * Create the outer join operator for this platform
+     */
+    protected static ExpressionOperator operatorOuterJoin() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.EqualOuterJoin);
+        Vector v = NonSynchronizedVector.newInstance(2);
+        v.addElement(" (+) = ");
+        result.printsAs(v);
+        result.bePostfix();
+        result.setNodeClass(RelationExpression.class);
+        return result;
+
+    }
+
     /**
      * INTERNAL:
      * Create the EXCEPT operator, MINUS in Oracle.
      */
-    public static ExpressionOperator exceptOperator() {
+    protected static ExpressionOperator exceptOperator() {
         ExpressionOperator exOperator = new ExpressionOperator();
         exOperator.setType(ExpressionOperator.FunctionOperator);
         exOperator.setSelector(ExpressionOperator.Except);
@@ -619,7 +626,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
      * INTERNAL:
      * Create the REGEXP_LIKE operator.
      */
-    public static ExpressionOperator regexpOperator() {
+    protected static ExpressionOperator regexpOperator() {
         ExpressionOperator result = new ExpressionOperator();
         result.setSelector(ExpressionOperator.Regexp);
         result.setType(ExpressionOperator.FunctionOperator);
@@ -637,6 +644,77 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return result;
     }
 
+    /**
+     * INTERNAL:
+     * Override the default locate operator
+     */
+    protected static ExpressionOperator operatorLocate() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.Locate);
+        Vector v = NonSynchronizedVector.newInstance(2);
+        v.addElement("INSTR(");
+        v.addElement(", ");
+        v.addElement(")");
+        result.printsAs(v);
+        result.bePrefix();
+        result.setNodeClass(RelationExpression.class);
+        return result;
+    }
+
+    /**
+     * INTERNAL:
+     * Override the default locate operator
+     */
+    protected static ExpressionOperator operatorLocate2() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.Locate2);
+        Vector v = NonSynchronizedVector.newInstance(2);
+        v.addElement("INSTR(");
+        v.addElement(", ");
+        v.addElement(", ");
+        v.addElement(")");
+        result.printsAs(v);
+        result.bePrefix();
+        result.setNodeClass(RelationExpression.class);
+        return result;
+    }
+
+    /**
+     *  Create the log operator for this platform
+     */
+    protected static ExpressionOperator logOperator() {
+        ExpressionOperator result = new ExpressionOperator();
+        result.setSelector(ExpressionOperator.Log);
+        Vector v = NonSynchronizedVector.newInstance(2);
+        v.addElement("LOG(10,");
+        v.addElement(")");
+        result.printsAs(v);
+        result.bePrefix();
+        result.setNodeClass(FunctionExpression.class);
+        return result;
+
+    }
+
+    /**
+     * INTERNAL:
+     * Oracle equivalent to DATENAME is TO_CHAR.
+     */
+    protected static ExpressionOperator oracleDateName() {
+        ExpressionOperator exOperator = new ExpressionOperator();
+        exOperator.setType(ExpressionOperator.FunctionOperator);
+        exOperator.setSelector(ExpressionOperator.DateName);
+        Vector v = org.eclipse.persistence.internal.helper.NonSynchronizedVector.newInstance(3);
+        v.add("TO_CHAR(");
+        v.add(", '");
+        v.add("')");
+        exOperator.printsAs(v);
+        exOperator.bePrefix();
+        int[] indices = { 1, 0 };
+        exOperator.setArgumentIndices(indices);
+        exOperator.setNodeClass(ClassConstants.FunctionExpression_Class);
+        return exOperator;
+    }
+
     /**
      * INTERNAL:
      * Used by derived platforms (Oracle8Platform and higher)
@@ -652,22 +730,6 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return true;
     }
 
-    /**
-     *  Create the log operator for this platform
-     */
-    protected ExpressionOperator logOperator() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.Log);
-        Vector v = NonSynchronizedVector.newInstance(2);
-        v.addElement("LOG(10,");
-        v.addElement(")");
-        result.printsAs(v);
-        result.bePrefix();
-        result.setNodeClass(FunctionExpression.class);
-        return result;
-
-    }
-
     /**
      *    Builds a table of maximum numeric values keyed on java class. This is used for type testing but
      * might also be useful to end users attempting to sanitize values.
@@ -730,56 +792,6 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return new ValueReadQuery("SELECT " + qualifiedSeqName + ".CURRVAL FROM DUAL");
     }
 
-    /**
-     * Create the outer join operator for this platform
-     */
-    protected ExpressionOperator operatorOuterJoin() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.EqualOuterJoin);
-        Vector v = NonSynchronizedVector.newInstance(2);
-        v.addElement(" (+) = ");
-        result.printsAs(v);
-        result.bePostfix();
-        result.setNodeClass(RelationExpression.class);
-        return result;
-
-    }
-
-    /**
-     * INTERNAL:
-     * Override the default locate operator
-     */
-    protected ExpressionOperator operatorLocate() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.Locate);
-        Vector v = NonSynchronizedVector.newInstance(2);
-        v.addElement("INSTR(");
-        v.addElement(", ");
-        v.addElement(")");
-        result.printsAs(v);
-        result.bePrefix();
-        result.setNodeClass(RelationExpression.class);
-        return result;
-    }
-
-    /**
-     * INTERNAL:
-     * Override the default locate operator
-     */
-    protected ExpressionOperator operatorLocate2() {
-        ExpressionOperator result = new ExpressionOperator();
-        result.setSelector(ExpressionOperator.Locate2);
-        Vector v = NonSynchronizedVector.newInstance(2);
-        v.addElement("INSTR(");
-        v.addElement(", ");
-        v.addElement(", ");
-        v.addElement(")");
-        result.printsAs(v);
-        result.bePrefix();
-        result.setNodeClass(RelationExpression.class);
-        return result;
-    }
-
     /**
      * INTERNAL:
      * Append the receiver's field 'NULL' constraint clause to a writer.
@@ -811,9 +823,9 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     @Override
-    public String getProcedureArgument(String name, Object parameter, Integer parameterType, 
+    public String getProcedureArgument(String name, Object parameter, ParameterType parameterType, 
             StoredProcedureCall call, AbstractSession session) {
-        if(name != null && DatasourceCall.IN.equals(parameterType) && !call.usesBinding(session)) {
+        if(name != null && ParameterType.IN.equals(parameterType) && !call.usesBinding(session)) {
             return name + "=>" + "?";
         }
         return "?";
@@ -906,21 +918,6 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         return true;
     }
 
-    /**
-     * Create the sysdate operator for this platform
-     */
-    protected ExpressionOperator todayOperator() {
-        return ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.Today, "SYSDATE");
-    }
-
-    protected ExpressionOperator currentDateOperator() {
-        return ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.CurrentDate, "TO_DATE(CURRENT_DATE)");
-    }
-
-    protected ExpressionOperator currentTimeOperator() {
-        return ExpressionOperator.simpleFunctionNoParentheses(ExpressionOperator.CurrentTime, "SYSDATE");
-    }
-
     /**
      * INTERNAL:
      * Indicates whether this Oracle platform can unwrap Oracle connection.
@@ -958,13 +955,15 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     protected String MIN_ROW = ") WHERE rnum > ";
     // Bug #453208
     protected String LOCK_START_PREFIX = " AND (";
+    protected String LOCK_START_PREFIX_WHERE = " WHERE (";
     protected String LOCK_START_SUFFIX = ") IN (";
-    protected String LOCK_END = ") FOR UPDATE";
+    protected String LOCK_END = " FOR UPDATE";
     protected String SELECT_ID_PREFIX = "SELECT ";
     protected String SELECT_ID_SUFFIX = " FROM (SELECT ";
     protected String FROM_ID = ", ROWNUM rnum  FROM (";
     protected String END_FROM_ID = ") ";
     protected String ORDER_BY_ID = " ORDER BY ";
+    protected String BRACKET_END = " ) ";
 
     /**
      * INTERNAL:
@@ -995,7 +994,7 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
                     // Workaround can exist for this specific case
                     Vector fields = new Vector();
                     statement.enableFieldAliasesCaching();
-                    String queryString = printOmittingForUpdate(statement, printer, fields);
+                    String queryString = printOmittingOrderByForUpdateUnion(statement, printer, fields);
                     duplicateCallParameters(call);
                     call.setFields(fields);
 
@@ -1030,6 +1029,13 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
                     printer.printParameter(DatabaseCall.MAXROW_FIELD);
                     printer.printString(MIN_ROW);
                     printer.printParameter(DatabaseCall.FIRSTRESULT_FIELD);
+                    printer.printString(BRACKET_END);
+                    try {
+                        statement.printSQLOrderByClause(printer);
+                        statement.printSQLUnionClause(printer);
+                    } catch (IOException exception) {
+                        throw ValidationException.fileError(exception);
+                    }
                     printer.printString(LOCK_END);
                 } else {
                     throw new UnsupportedOperationException(ExceptionLocalization.buildMessage("ora_pessimistic_locking_with_rownum"));
@@ -1067,20 +1073,31 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
         ArrayList newParameterList = new ArrayList(call.getParameters());
         newParameterList.addAll(call.getParameters());
         call.setParameters(newParameterList);
-        ArrayList<Integer> newParameterTypesList = new ArrayList(call.getParameterTypes());
+        ArrayList<ParameterType> newParameterTypesList = new ArrayList(call.getParameterTypes());
         newParameterTypesList.addAll(call.getParameterTypes());
         call.setParameterTypes(newParameterTypesList);
     }
 
     @SuppressWarnings("unchecked")
-    private String printOmittingForUpdate(SQLSelectStatement statement, ExpressionSQLPrinter printer, Vector fields) {
+    private String printOmittingOrderByForUpdateUnion(SQLSelectStatement statement, ExpressionSQLPrinter printer, Vector fields) {
         boolean originalShouldPrintForUpdate = this.shouldPrintForUpdateClause;
         Writer originalWriter = printer.getWriter();
+        Vector selectFields = null;
 
         this.shouldPrintForUpdateClause = false;
         printer.setWriter(new StringWriter());
 
-        fields.addAll(statement.printSQL(printer));
+        try {
+            selectFields = statement.printSQLSelect(printer);
+            statement.printSQLWhereKeyWord(printer);
+            statement.printSQLWhereClause(printer);
+            statement.printSQLHierarchicalQueryClause(printer);
+            statement.printSQLGroupByClause(printer);
+            statement.printSQLHavingClause(printer);
+        } catch (IOException exception) {
+            throw ValidationException.fileError(exception);
+        }
+        fields.addAll(selectFields);
         String query = printer.getWriter().toString();
 
         this.shouldPrintForUpdateClause = originalShouldPrintForUpdate;
@@ -1090,7 +1107,11 @@ public class OraclePlatform extends org.eclipse.persistence.platform.database.Da
     }
 
     private void printLockStartWithPrimaryKeyFields(SQLSelectStatement statement, ExpressionSQLPrinter printer) {
-        printer.printString(LOCK_START_PREFIX);
+        if (statement.getWhereClause() == null) {
+            printer.printString(LOCK_START_PREFIX_WHERE);
+        } else {
+            printer.printString(LOCK_START_PREFIX);
+        }
 
         Iterator<DatabaseField> iterator = statement.getQuery().getDescriptor().getPrimaryKeyFields().iterator();
         while (iterator.hasNext()) {
