@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2022 IBM Corporation. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Public License v. 2.0 which is available at
@@ -64,6 +65,7 @@ import java.sql.Struct;
 import java.util.List;
 import java.util.ListIterator;
 
+import org.eclipse.persistence.internal.databaseaccess.DatasourceCall.ParameterType;
 import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.helper.DatabaseField;
 import org.eclipse.persistence.internal.helper.DatabaseType;
@@ -300,8 +302,15 @@ public enum JDBCTypes implements JDBCType {
         }
 
         @Override
+        @Deprecated
         public void logParameter(StringBuilder sb, Integer direction, PLSQLargument arg,
                 AbstractRecord translationRow, DatabasePlatform platform) {
+            logParameter(sb, ParameterType.valueOf(direction), arg, translationRow, platform);
+        }
+
+        @Override
+        public void logParameter(StringBuilder sb, ParameterType direction, PLSQLargument arg,
+                AbstractRecord translationRow, DatabasePlatform platform) {
             databaseTypeHelper.logParameter(sb, direction, arg, translationRow, platform);
         }
 
